@@ -7,7 +7,7 @@
  * 
  * Copyright (c) 2005 James Su <suzhe@tsinghua.org.cn>
  *
- * $Id: scim_pinyin_imengine.cpp,v 1.1.1.1 2005/01/06 13:31:02 suzhe Exp $
+ * $Id: scim_pinyin_imengine.cpp,v 1.6 2005/08/06 15:19:01 suzhe Exp $
  *
  */
 
@@ -51,9 +51,18 @@
 #define scim_imengine_module_create_factory pinyin_LTX_scim_imengine_module_create_factory
 
 
-#define SCIM_PROP_STATUS                                                     "/IMEngine/Pinyin/Status"
-#define SCIM_PROP_LETTER                                                     "/IMEngine/Pinyin/Letter"
-#define SCIM_PROP_PUNCT                                                      "/IMEngine/Pinyin/Punct"
+#define SCIM_PROP_STATUS                            "/IMEngine/Pinyin/Status"
+#define SCIM_PROP_LETTER                            "/IMEngine/Pinyin/Letter"
+#define SCIM_PROP_PUNCT                             "/IMEngine/Pinyin/Punct"
+
+#define SCIM_PROP_PINYIN_SCHEME                     "/IMEngine/Pinyin/PinyinScheme"
+#define SCIM_PROP_PINYIN_SCHEME_QUAN_PIN            "/IMEngine/Pinyin/PinyinScheme/QuanPin"
+#define SCIM_PROP_PINYIN_SCHEME_SP_STONE            "/IMEngine/Pinyin/PinyinScheme/SP-STONE"
+#define SCIM_PROP_PINYIN_SCHEME_SP_ZRM              "/IMEngine/Pinyin/PinyinScheme/SP-ZRM"
+#define SCIM_PROP_PINYIN_SCHEME_SP_MS               "/IMEngine/Pinyin/PinyinScheme/SP-MS"
+#define SCIM_PROP_PINYIN_SCHEME_SP_ZIGUANG          "/IMEngine/Pinyin/PinyinScheme/SP-ZIGUANG"
+#define SCIM_PROP_PINYIN_SCHEME_SP_ABC              "/IMEngine/Pinyin/PinyinScheme/SP-ABC"
+#define SCIM_PROP_PINYIN_SCHEME_SP_LIUSHI           "/IMEngine/Pinyin/PinyinScheme/SP-LIUSHI"
 
 #ifndef SCIM_PINYIN_DATADIR
     #define SCIM_PINYIN_DATADIR "/usr/share/scim/pinyin"
@@ -80,9 +89,18 @@ static IMEngineFactoryPointer _scim_pinyin_factory (0);
 
 static ConfigPointer _scim_config (0);
 
-static Property _status_property (SCIM_PROP_STATUS, "");
-static Property _letter_property (SCIM_PROP_LETTER, "");
-static Property _punct_property  (SCIM_PROP_PUNCT, "");
+static Property _status_property   (SCIM_PROP_STATUS, "");
+static Property _letter_property   (SCIM_PROP_LETTER, "");
+static Property _punct_property    (SCIM_PROP_PUNCT, "");
+
+static Property _pinyin_scheme_property     (SCIM_PROP_PINYIN_SCHEME, "全");
+static Property _pinyin_quan_pin_property   (SCIM_PROP_PINYIN_SCHEME_QUAN_PIN,   "全拼");
+static Property _pinyin_sp_stone_property   (SCIM_PROP_PINYIN_SCHEME_SP_STONE,   "双拼-中文之星/四通利方");
+static Property _pinyin_sp_zrm_property     (SCIM_PROP_PINYIN_SCHEME_SP_ZRM,     "双拼-自然码");
+static Property _pinyin_sp_ms_property      (SCIM_PROP_PINYIN_SCHEME_SP_MS,      "双拼-微软拼音");
+static Property _pinyin_sp_ziguang_property (SCIM_PROP_PINYIN_SCHEME_SP_ZIGUANG, "双拼-紫光拼音");
+static Property _pinyin_sp_abc_property     (SCIM_PROP_PINYIN_SCHEME_SP_ABC,     "双拼-智能ABC");
+static Property _pinyin_sp_liushi_property  (SCIM_PROP_PINYIN_SCHEME_SP_LIUSHI,  "双拼-刘氏");
 
 extern "C" {
     void scim_module_init (void)
@@ -101,9 +119,9 @@ extern "C" {
     {
         _status_property.set_tip (_("The status of the current input method. Click to change it."));
         _letter_property.set_tip (_("The input mode of the letters. Click to toggle between half and full."));
-	_letter_property.set_label (_("Full/Half Letter"));
+        _letter_property.set_label (_("Full/Half Letter"));
         _punct_property.set_tip (_("The input mode of the puncutations. Click to toggle between half and full."));
-	_punct_property.set_label (_("Full/Half Punct"));
+        _punct_property.set_label (_("Full/Half Punct"));
 
         _status_property.set_label ("英");
         _letter_property.set_icon (SCIM_HALF_LETTER_ICON);
@@ -130,6 +148,7 @@ extern "C" {
 // implementation of PinyinFactory
 PinyinFactory::PinyinFactory (const ConfigPointer &config)
     : m_config (config),
+      m_pinyin_parser (0),
       m_match_longer_phrase (false),
       m_auto_combine_phrase (false),
       m_auto_fill_preedit (false),
@@ -137,6 +156,8 @@ PinyinFactory::PinyinFactory (const ConfigPointer &config)
       m_show_all_keys (false),
       m_user_data_binary (true),
       m_valid (false),
+      m_shuang_pin (false),
+      m_shuang_pin_scheme (SCIM_SHUANG_PIN_STONE),
       m_last_time ((time_t)0),
       m_save_period ((time_t)300),
       m_dynamic_sensitivity (6),
@@ -392,12 +413,26 @@ PinyinFactory::init ()
                               String ("Control+d"));
 
         scim_string_to_key_list (m_disable_phrase_keys, str);
+
+        // Load Pinyin Scheme config
+        m_shuang_pin =
+            m_config->read (String (SCIM_CONFIG_IMENGINE_PINYIN_SHUANG_PIN),
+                            m_shuang_pin);
+
+        if (m_shuang_pin) {
+            int tmp;
+            tmp = m_config->read (String (SCIM_CONFIG_IMENGINE_PINYIN_SHUANG_PIN_SCHEME),
+                                  (int) m_shuang_pin_scheme);
+
+            if (tmp >=0 && tmp <= SCIM_SHUANG_PIN_LIUSHI)
+                m_shuang_pin_scheme = static_cast<PinyinShuangPinScheme> (tmp);
+        }
     }
 
     if (m_save_period > 0 && m_save_period < 30)
         m_save_period = 30;
 
-    m_name = utf8_mbstowcs (_("Smart Pinyin"));
+    m_name = utf8_mbstowcs ("智能拼音");
 
     m_pinyin_global.toggle_tone (tone);
     m_pinyin_global.toggle_incomplete (incomplete);
@@ -477,9 +512,23 @@ PinyinFactory::init ()
     if (is_user_special_table)
         m_special_table.load (is_user_special_table);
 
+    init_pinyin_parser ();
+
     return true;
 }
 
+void
+PinyinFactory::init_pinyin_parser ()
+{
+    if (m_pinyin_parser)
+        delete m_pinyin_parser;
+
+    if (m_shuang_pin)
+        m_pinyin_parser = new PinyinShuangPinParser (m_shuang_pin_scheme);
+    else
+        m_pinyin_parser = new PinyinDefaultParser ();
+}
+
 PinyinFactory::~PinyinFactory ()
 {
     if (m_valid)
@@ -641,6 +690,7 @@ PinyinInstance::PinyinInstance (PinyinFactory *factory,
       m_focused (false),
       m_simplified (true),
       m_traditional (true),
+      m_lookup_table_def_page_size (9),
       m_keys_caret (0),
       m_lookup_caret (0),
       m_iconv (encoding)
@@ -651,20 +701,6 @@ PinyinInstance::PinyinInstance (PinyinFactory *factory,
     m_full_width_letter [0] = false;
     m_full_width_letter [1] = false;
 
-    std::vector<WideString> candidate_labels;
-    char buf [2] = { 0, 0 };
-
-    for (int i=0; i<9; i++) {
-        buf [0] = '1' + i;
-        candidate_labels.push_back (utf8_mbstowcs (buf));
-    }
-
-    candidate_labels.push_back (utf8_mbstowcs ("0"));
-
-    m_lookup_table.set_page_size (10);
-    m_lookup_table.set_candidate_labels (candidate_labels);
-    m_lookup_table.show_cursor ();
-
     if (m_factory->valid () && m_pinyin_global) {
         m_pinyin_table = m_pinyin_global->get_pinyin_table ();
         m_sys_phrase_lib = m_pinyin_global->get_sys_phrase_lib ();
@@ -686,6 +722,8 @@ PinyinInstance::PinyinInstance (PinyinFactory *factory,
     }
 
     m_reload_signal_connection = m_factory->m_config->signal_connect_reload (slot (this, &PinyinInstance::reload_config));
+
+    init_lookup_table_labels ();
 }
 
 PinyinInstance::~PinyinInstance ()
@@ -746,18 +784,20 @@ PinyinInstance::process_key_event (const KeyEvent& key)
             return true;
         }
 
-        //go into english mode
-        if ((!m_inputed_string.length () && key.code == SCIM_KEY_v && key.mask == 0) ||
-            is_english_mode ()) {
-            return english_mode_process_key_event (key);
-        }
-
-        //go into english mode
-        if ((!m_inputed_string.length () &&
-             key.code == SCIM_KEY_i && key.mask == 0 &&
-             m_factory->m_special_table.valid ()) ||
-            is_special_mode ()) {
-            return special_mode_process_key_event (key);
+        if (!m_factory->m_shuang_pin) {
+            //go into english mode
+            if ((!m_inputed_string.length () && key.code == SCIM_KEY_v && key.mask == 0) ||
+                is_english_mode ()) {
+                return english_mode_process_key_event (key);
+            }
+ 
+            //go into english mode
+            if ((!m_inputed_string.length () &&
+                 key.code == SCIM_KEY_i && key.mask == 0 &&
+                 m_factory->m_special_table.valid ()) ||
+                is_special_mode ()) {
+                return special_mode_process_key_event (key);
+            }
         }
 
         //caret left
@@ -813,11 +853,18 @@ PinyinInstance::process_key_event (const KeyEvent& key)
         }
 
         //select lookup table
-        if (key.code >= SCIM_KEY_0 && key.code <= SCIM_KEY_9 && key.mask == 0) {
-            int index;
-            if (key.code == SCIM_KEY_0) index = 9;
-            else index = key.code - SCIM_KEY_1;
-            if (lookup_select (index)) return true;
+        if (m_pinyin_global->use_tone ()) {
+            if (((key.code >= SCIM_KEY_6 && key.code <= SCIM_KEY_9) || key.code == SCIM_KEY_0) && key.mask == 0) {
+                int index;
+                if (key.code == SCIM_KEY_0) index = 4;
+                else index = key.code - SCIM_KEY_6;
+                if (lookup_select (index)) return true;
+            }
+        } else {
+            if (key.code >= SCIM_KEY_1 && key.code <= SCIM_KEY_9 && key.mask == 0) {
+                int index = key.code - SCIM_KEY_1;
+                if (lookup_select (index)) return true;
+            }
         }
 
         //space hit
@@ -833,11 +880,11 @@ PinyinInstance::process_key_event (const KeyEvent& key)
             return disable_phrase();
         }
 
-        if ((key.mask & (~ (SCIM_KEY_ShiftMask + SCIM_KEY_LockMask))) == 0)
+        if ((key.mask & (~ (SCIM_KEY_ShiftMask + SCIM_KEY_CapsLockMask))) == 0)
             return insert (key.get_ascii_code ());
     }
 
-    if ((key.mask & (~ (SCIM_KEY_ShiftMask + SCIM_KEY_LockMask))) == 0)
+    if ((key.mask & (~ (SCIM_KEY_ShiftMask + SCIM_KEY_CapsLockMask))) == 0)
         return post_process (key.get_ascii_code ());
 
     return false;
@@ -879,6 +926,20 @@ PinyinInstance::move_preedit_caret (unsigned int /*pos*/)
 void
 PinyinInstance::reset ()
 {
+    String encoding = get_encoding ();
+
+    m_iconv.set_encoding (encoding);
+
+    if (encoding == "GBK" || encoding == "GB2312") {
+        m_simplified = true;
+        m_traditional = false;
+        m_chinese_iconv.set_encoding ("GB2312");
+    } else if (encoding == "BIG5" || encoding == "BIG5-HKSCS") {
+        m_simplified = false;
+        m_traditional = true;
+        m_chinese_iconv.set_encoding ("BIG5");
+    }
+
     m_double_quotation_state = false;
     m_single_quotation_state = false;
 
@@ -916,13 +977,15 @@ PinyinInstance::focus_in ()
     hide_preedit_string ();
     hide_aux_string ();
 
+    init_lookup_table_labels ();
+
     if (!is_english_mode ()) {
         refresh_preedit_string ();
         refresh_preedit_caret ();
         refresh_aux_string ();
 
         if (m_lookup_table.number_of_candidates ()) {
-            m_lookup_table.set_page_size (10);
+            m_lookup_table.set_page_size (m_lookup_table_def_page_size);
             show_lookup_table ();
             update_lookup_table (m_lookup_table);
         }
@@ -940,6 +1003,8 @@ PinyinInstance::focus_out ()
 void
 PinyinInstance::trigger_property (const String &property)
 {
+    bool update_pinyin_scheme = false;
+
     if (property == SCIM_PROP_STATUS) {
         if (m_forward) {
             m_simplified = true;
@@ -974,6 +1039,41 @@ PinyinInstance::trigger_property (const String &property)
         m_full_width_punctuation [which] = !m_full_width_punctuation [which];
 
         refresh_punct_property ();
+    } else if (property == SCIM_PROP_PINYIN_SCHEME_QUAN_PIN) {
+        m_factory->m_shuang_pin = false;
+        update_pinyin_scheme = true;
+    } else if (property == SCIM_PROP_PINYIN_SCHEME_SP_STONE) {
+        m_factory->m_shuang_pin = true;
+        m_factory->m_shuang_pin_scheme = SCIM_SHUANG_PIN_STONE;
+        update_pinyin_scheme = true;
+    } else if (property == SCIM_PROP_PINYIN_SCHEME_SP_ZRM) {
+        m_factory->m_shuang_pin = true;
+        m_factory->m_shuang_pin_scheme = SCIM_SHUANG_PIN_ZRM;
+        update_pinyin_scheme = true;
+    } else if (property == SCIM_PROP_PINYIN_SCHEME_SP_MS) {
+        m_factory->m_shuang_pin = true;
+        m_factory->m_shuang_pin_scheme = SCIM_SHUANG_PIN_MS;
+        update_pinyin_scheme = true;
+    } else if (property == SCIM_PROP_PINYIN_SCHEME_SP_ZIGUANG) {
+        m_factory->m_shuang_pin = true;
+        m_factory->m_shuang_pin_scheme = SCIM_SHUANG_PIN_ZIGUANG;
+        update_pinyin_scheme = true;
+    } else if (property == SCIM_PROP_PINYIN_SCHEME_SP_ABC) {
+        m_factory->m_shuang_pin = true;
+        m_factory->m_shuang_pin_scheme = SCIM_SHUANG_PIN_ABC;
+        update_pinyin_scheme = true;
+    } else if (property == SCIM_PROP_PINYIN_SCHEME_SP_LIUSHI) {
+        m_factory->m_shuang_pin = true;
+        m_factory->m_shuang_pin_scheme = SCIM_SHUANG_PIN_LIUSHI;
+        update_pinyin_scheme = true;
+    }
+
+    if (update_pinyin_scheme) {
+        m_factory->init_pinyin_parser ();
+        refresh_pinyin_scheme_property ();
+        reset ();
+        m_factory->m_config->write (String (SCIM_CONFIG_IMENGINE_PINYIN_SHUANG_PIN), m_factory->m_shuang_pin);
+        m_factory->m_config->write (String (SCIM_CONFIG_IMENGINE_PINYIN_SHUANG_PIN_SCHEME), (int) m_factory->m_shuang_pin_scheme);
     }
 }
 
@@ -1037,12 +1137,22 @@ void
 PinyinInstance::initialize_all_properties ()
 {
     PropertyList proplist;
+
+    proplist.push_back (_pinyin_scheme_property);
+    proplist.push_back (_pinyin_quan_pin_property);
+    proplist.push_back (_pinyin_sp_stone_property);
+    proplist.push_back (_pinyin_sp_zrm_property);
+    proplist.push_back (_pinyin_sp_ms_property);
+    proplist.push_back (_pinyin_sp_ziguang_property);
+    proplist.push_back (_pinyin_sp_abc_property);
+    proplist.push_back (_pinyin_sp_liushi_property);
     proplist.push_back (_status_property);
     proplist.push_back (_letter_property);
     proplist.push_back (_punct_property);
 
     register_properties (proplist);
     refresh_all_properties ();
+    refresh_pinyin_scheme_property ();
 }
 
 void
@@ -1086,6 +1196,70 @@ PinyinInstance::refresh_punct_property ()
     update_property (_punct_property);
 }
 
+void
+PinyinInstance::refresh_pinyin_scheme_property ()
+{
+    String tip;
+
+    if (m_factory->m_shuang_pin) {
+        switch (m_factory->m_shuang_pin_scheme) {
+            case SCIM_SHUANG_PIN_STONE:
+                tip = _pinyin_sp_stone_property.get_label ();
+                break;
+            case SCIM_SHUANG_PIN_ZRM:
+                tip = _pinyin_sp_zrm_property.get_label ();
+                break;
+            case SCIM_SHUANG_PIN_MS:
+                tip = _pinyin_sp_ms_property.get_label ();
+                break;
+            case SCIM_SHUANG_PIN_ZIGUANG:
+                tip = _pinyin_sp_ziguang_property.get_label ();
+                break;
+            case SCIM_SHUANG_PIN_ABC:
+                tip = _pinyin_sp_abc_property.get_label ();
+                break;
+            case SCIM_SHUANG_PIN_LIUSHI:
+                tip = _pinyin_sp_liushi_property.get_label ();
+                break;
+        }
+        _pinyin_scheme_property.set_label ("双");
+    } else {
+        tip = _pinyin_quan_pin_property.get_label ();
+        _pinyin_scheme_property.set_label ("全");
+    }
+
+    _pinyin_scheme_property.set_tip (tip);
+    update_property (_pinyin_scheme_property);
+}
+
+void
+PinyinInstance::init_lookup_table_labels ()
+{
+    std::vector<WideString> candidate_labels;
+    char buf [2] = { 0, 0 };
+
+    if (m_pinyin_global->use_tone ()) {
+        for (int i=5; i<9; i++) {
+            buf [0] = '1' + i;
+            candidate_labels.push_back (utf8_mbstowcs (buf));
+        }
+
+        buf [0] = '0';
+        candidate_labels.push_back (utf8_mbstowcs (buf));
+    } else {
+        for (int i=0; i<9; i++) {
+            buf [0] = '1' + i;
+            candidate_labels.push_back (utf8_mbstowcs (buf));
+        }
+    }
+
+    m_lookup_table_def_page_size = (int) candidate_labels.size ();
+
+    m_lookup_table.set_page_size (m_lookup_table_def_page_size);
+    m_lookup_table.set_candidate_labels (candidate_labels);
+    m_lookup_table.show_cursor ();
+}
+
 bool
 PinyinInstance::caret_left (bool home)
 {
@@ -1145,6 +1319,21 @@ PinyinInstance::caret_right (bool end)
     return false;
 }
 
+bool
+PinyinInstance::validate_insert_key (char key)
+{
+    if (m_pinyin_global->use_tone () && key >= '1' && key <= '5')
+        return true;
+
+    if (m_factory->m_shuang_pin && key == ';')
+        return true;
+
+    if ((key >= 'a' && key <= 'z') || key == '\'')
+        return true;
+
+    return false;
+}
+
 bool
 PinyinInstance::insert (char key)
 {
@@ -1154,10 +1343,7 @@ PinyinInstance::insert (char key)
     String old_inputed_string = m_inputed_string;
     PinyinKeyExactEqualTo equal;
 
-    if ((key >= 'a' && key <= 'z') || (key == '\'') ||
-        (m_pinyin_global->use_tone () && 
-         (key == '!' || key == '@' || key == '#' || key == '$' || key == '%'))) {
-
+    if (validate_insert_key (key)) {
         int inputed_caret = calc_inputed_caret ();
 
         if ((m_parsed_keys.size () != 0 &&
@@ -1166,30 +1352,14 @@ PinyinInstance::insert (char key)
              m_inputed_string.length () > SCIM_PINYIN_KEY_MAXLEN))
             return true;
 
-        if (inputed_caret == 0 && 
-            (key == '\'' ||
-             key == '!' ||
-             key == '@' ||
-             key == '#' ||
-             key == '$' ||
-             key == '%'))
+        if (inputed_caret == 0 && ((key >= '1' && key <= '5') || key == '\'' || key == ';'))
             return post_process (key);
 
         String::iterator i = m_inputed_string.begin () + inputed_caret;
 
-        if (key >= 'a' && key <= 'z')
+        if (key != '\'') {
             m_inputed_string.insert (i, key);
-        else if (key == '!')
-            m_inputed_string.insert (i, '1');
-        else if (key == '@')
-            m_inputed_string.insert (i, '2');
-        else if (key == '#')
-            m_inputed_string.insert (i, '3');
-        else if (key == '$')
-            m_inputed_string.insert (i, '4');
-        else if (key == '%')
-            m_inputed_string.insert (i, '5');
-        else if (key == '\'') {
+        } else {
             // Don't insert more than one split chars in one place.
             if ((i != m_inputed_string.begin () && *(i-1) == '\'') ||
                 (i != m_inputed_string.end () && *i == '\''))
@@ -1425,7 +1595,7 @@ PinyinInstance::lookup_cursor_up ()
 {
     if (m_inputed_string.length () && m_lookup_table.number_of_candidates ()) {
         m_lookup_table.cursor_up ();
-        m_lookup_table.set_page_size (10);
+        m_lookup_table.set_page_size (m_lookup_table_def_page_size);
         update_lookup_table (m_lookup_table);
         return true;
     }
@@ -1437,7 +1607,7 @@ PinyinInstance::lookup_cursor_down ()
 {
     if (m_inputed_string.length () && m_lookup_table.number_of_candidates ()) {
         m_lookup_table.cursor_down ();
-        m_lookup_table.set_page_size (10);
+        m_lookup_table.set_page_size (m_lookup_table_def_page_size);
         update_lookup_table (m_lookup_table);
         return true;
     }
@@ -1449,7 +1619,7 @@ PinyinInstance::lookup_page_up ()
 {
     if (m_inputed_string.length () && m_lookup_table.number_of_candidates ()) {
         m_lookup_table.page_up ();
-        m_lookup_table.set_page_size (10);
+        m_lookup_table.set_page_size (m_lookup_table_def_page_size);
         update_lookup_table (m_lookup_table);
         return true;
     }
@@ -1461,7 +1631,7 @@ PinyinInstance::lookup_page_down ()
 {
     if (m_inputed_string.length () && m_lookup_table.number_of_candidates ()) {
         m_lookup_table.page_down ();
-        m_lookup_table.set_page_size (10);
+        m_lookup_table.set_page_size (m_lookup_table_def_page_size);
         update_lookup_table (m_lookup_table);
         return true;
     }
@@ -1713,9 +1883,9 @@ PinyinInstance::refresh_preedit_caret ()
 void
 PinyinInstance::calc_parsed_keys ()
 {
-    PinyinKey::parse_pinyin_key (m_pinyin_global->get_pinyin_validator (),
-                                                m_parsed_keys,
-                                                m_inputed_string.c_str ());
+    m_factory->m_pinyin_parser->parse (m_pinyin_global->get_pinyin_validator (),
+                                       m_parsed_keys,
+                                       m_inputed_string.c_str ());
 }
 
 void
@@ -1724,7 +1894,7 @@ PinyinInstance::calc_lookup_table (int invalid_pos,
                                          PhraseVector *matched_phrases)
 {
     m_lookup_table.clear ();
-    m_lookup_table.set_page_size (10);
+    m_lookup_table.set_page_size (m_lookup_table_def_page_size);
 
     if (combined) *combined = WideString ();
     if (matched_phrases) matched_phrases->clear ();
@@ -2302,14 +2472,14 @@ PinyinInstance::english_mode_process_key_event (const KeyEvent &key)
     }
     //enter or space hit, commit string
     else if ((key.code == SCIM_KEY_Return || key.code == SCIM_KEY_space) &&
-             (key.mask & (~ (SCIM_KEY_ShiftMask + SCIM_KEY_LockMask))) == 0) {
+             (key.mask & (~ (SCIM_KEY_ShiftMask + SCIM_KEY_CapsLockMask))) == 0) {
         WideString str = m_converted_string.substr (1);
         if (str.length ())
             commit_string (str);
         m_converted_string.clear ();
     }
     //insert normal keys
-    else if ((key.mask & (~ (SCIM_KEY_ShiftMask + SCIM_KEY_LockMask))) == 0) {
+    else if ((key.mask & (~ (SCIM_KEY_ShiftMask + SCIM_KEY_CapsLockMask))) == 0) {
         char ch = key.get_ascii_code ();
         if ((ispunct (ch) && m_full_width_punctuation [1]) ||
             (isalnum (ch) && m_full_width_letter [1])) {
@@ -2323,7 +2493,7 @@ PinyinInstance::english_mode_process_key_event (const KeyEvent &key)
         }
     }
     //forward special keys directly.
-    else if ((key.mask & (~ (SCIM_KEY_ShiftMask + SCIM_KEY_LockMask)))) 
+    else if ((key.mask & (~ (SCIM_KEY_ShiftMask + SCIM_KEY_CapsLockMask)))) 
         return false;
 
     if (m_converted_string.length ()) {
@@ -2386,11 +2556,18 @@ PinyinInstance::special_mode_process_key_event (const KeyEvent &key)
         if (lookup_page_down ()) return true;
 
     //select lookup table
-    if (key.code >= SCIM_KEY_0 && key.code <= SCIM_KEY_9 && key.mask == 0) {
-        int index;
-        if (key.code == SCIM_KEY_0) index = 9;
-        else index = key.code - SCIM_KEY_1;
-        if (special_mode_lookup_select (index)) return true;
+    if (m_pinyin_global->use_tone ()) {
+        if (((key.code >= SCIM_KEY_6 && key.code <= SCIM_KEY_9) || key.code == SCIM_KEY_0) && key.mask == 0) {
+            int index;
+            if (key.code == SCIM_KEY_0) index = 4;
+            else index = key.code - SCIM_KEY_6;
+            if (special_mode_lookup_select (index)) return true;
+        }
+    } else {
+        if (key.code >= SCIM_KEY_1 && key.code <= SCIM_KEY_9 && key.mask == 0) {
+            int index = key.code - SCIM_KEY_1;
+            if (special_mode_lookup_select (index)) return true;
+        }
     }
 
     //backspace key
@@ -2402,7 +2579,7 @@ PinyinInstance::special_mode_process_key_event (const KeyEvent &key)
     }
     //enter or space hit, commit string
     else if ((key.code == SCIM_KEY_space || key.code == SCIM_KEY_Return) &&
-             (key.mask & (~ (SCIM_KEY_ShiftMask + SCIM_KEY_LockMask))) == 0) {
+             (key.mask & (~ (SCIM_KEY_ShiftMask + SCIM_KEY_CapsLockMask))) == 0) {
 
         if (m_lookup_table.number_of_candidates ())
             commit_string (m_lookup_table.get_candidate (m_lookup_table.get_cursor_pos ()));
@@ -2413,7 +2590,7 @@ PinyinInstance::special_mode_process_key_event (const KeyEvent &key)
         m_converted_string.clear ();
     }
     //insert normal keys
-    else if ((key.mask & (~ (SCIM_KEY_ShiftMask + SCIM_KEY_LockMask))) == 0 &&
+    else if ((key.mask & (~ (SCIM_KEY_ShiftMask + SCIM_KEY_CapsLockMask))) == 0 &&
               key.code != 0 &&
               m_inputed_string.length () <= m_factory->m_special_table.get_max_key_length ()) {
         char ch = key.get_ascii_code ();
@@ -2426,7 +2603,7 @@ PinyinInstance::special_mode_process_key_event (const KeyEvent &key)
         }
     }
     //forward special keys directly.
-    else if ((key.mask & (~ (SCIM_KEY_ShiftMask + SCIM_KEY_LockMask)))) 
+    else if ((key.mask & (~ (SCIM_KEY_ShiftMask + SCIM_KEY_CapsLockMask)))) 
         return false;
 
     if (m_inputed_string.length ()) {
@@ -2455,7 +2632,7 @@ void
 PinyinInstance::special_mode_refresh_lookup_table ()
 {
     m_lookup_table.clear ();
-    m_lookup_table.set_page_size (10);
+    m_lookup_table.set_page_size (m_lookup_table_def_page_size);
 
     if (m_inputed_string.length () > 1) {
         std::vector<WideString> result;
