@@ -7,7 +7,7 @@
  * 
  * Copyright (c) 2005 James Su <suzhe@tsinghua.org.cn>
  *
- * $Id: scim_pinyin_imengine_setup.cpp,v 1.1.1.1 2005/01/06 13:31:00 suzhe Exp $
+ * $Id: scim_pinyin_imengine_setup.cpp,v 1.2 2005/03/03 23:53:27 suzhe Exp $
  *
  */
 
@@ -1290,7 +1290,7 @@ static void
 __on_ambiguities_toggled               (GtkToggleButton *togglebutton,
                                         gpointer         user_data)
 {
-    int  index = (int) user_data;
+    size_t index = (size_t) user_data;
 
     if (index >= 0 && index <= SCIM_PINYIN_AmbLast) {
         __config_ambiguities [index] = gtk_toggle_button_get_active (togglebutton);
