@@ -7,7 +7,7 @@
  * 
  * Copyright (c) 2005 James Su <suzhe@tsinghua.org.cn>
  *
- * $Id: scim_pinyin_phrase.cpp,v 1.1.1.1 2005/01/06 13:30:58 suzhe Exp $
+ * $Id: scim_pinyin_phrase.cpp,v 1.2 2005/08/06 12:06:49 suzhe Exp $
  *
  */
 #define Uses_STL_FUNCTIONAL
@@ -180,7 +180,7 @@ PinyinPhraseLib::PinyinPhraseLib (const PinyinCustomSettings &custom,
 	  m_pinyin_phrase_less_by_offset (this, custom),
 	  m_pinyin_phrase_equal_by_offset (this, custom)
 {
-	if (!m_validator) m_validator = &scim_default_pinyin_validator;
+	if (!m_validator) m_validator = PinyinValidator::get_default_pinyin_validator ();
 
 	load_lib (libfile, pylibfile, idxfile);
 }
@@ -198,7 +198,7 @@ PinyinPhraseLib::PinyinPhraseLib (const PinyinCustomSettings &custom,
 	  m_pinyin_phrase_less_by_offset (this, custom),
 	  m_pinyin_phrase_equal_by_offset (this, custom)
 {
-	if (!m_validator) m_validator = &scim_default_pinyin_validator;
+	if (!m_validator) m_validator = PinyinValidator::get_default_pinyin_validator ();
 
 	input (is_lib, is_pylib, is_idx);
 }
@@ -414,7 +414,7 @@ PinyinPhraseLib::update_custom_settings (const PinyinCustomSettings &custom,
 	m_validator = validator;
 
 	if (!m_validator)
-		m_validator = &scim_default_pinyin_validator;
+		m_validator = PinyinValidator::get_default_pinyin_validator ();
 
 	sort_phrase_tables ();
 }
@@ -479,8 +479,11 @@ PinyinPhraseLib::find_phrases (PhraseVector &vec,
 							   bool noshorter,
 							   bool nolonger)
 {
-	PinyinKeyVector pykeys;
-	PinyinKey::parse_pinyin_key (*m_validator, pykeys, keys);
+	PinyinParsedKeyVector pykeys;
+	PinyinDefaultParser parser;
+
+	parser.parse (*m_validator, pykeys, keys);
+
 	return find_phrases (vec, pykeys, noshorter, nolonger);
 }
 
